<?php
/*
Plugin Name: Pushover Notifications for Jetpack
Plugin URI: http://wp-push.com
Description: Adds Pushover Notifications for Jetpack
Version: 1.0.1
Author: Chris Klosowski
Author URI: http://wp-push.com
Text Domain: ckpn-jp
*/

// Define the plugin path
define( 'CKPN_JP_PATH', plugin_dir_path( __FILE__ ) );

define( 'CKPN_JP_TEXT_DOMAIN' , 'ckpn-jp' );
// plugin version
define( 'CKPN_JP_VERSION', '1.0' );

// Define the URL to the plugin folder
define( 'CKPN_JP_FOLDER', dirname( plugin_basename( __FILE__ ) ) );
define( 'CKPN_JP_URL', plugins_url( '', __FILE__ ) );

include_once ABSPATH . 'wp-admin/includes/plugin.php';


class CKPushoverNotificationsJP {
	private static $CKPN_JP_instance;

	private function __construct() {
		if ( !$this->checkCoreVersion() ) {
			add_action( 'admin_notices', array( $this, 'core_out_of_date_nag' ) );
		} else if ( ! $this->check_jetpack_active() ) {
			add_action( 'admin_notices', array( $this, 'jetpack_not_active' ) );
		} else if ( ! $this->check_stats_active() ) {
			add_action( 'admin_notices', array( $this, 'stats_not_active' ) );
		} else {
			add_action( 'init', array( $this, 'determine_cron_schedule' ) );
		}
	}

	public static function getInstance() {
		if ( !self::$CKPN_JP_instance ) {
			self::$CKPN_JP_instance = new CKPushoverNotificationsJP();
		}

		return self::$CKPN_JP_instance;
	}

	private function checkCoreVersion() {
		if ( !is_plugin_active( 'pushover-notifications/pushover-notifications.php' ) )
			return false;

		// Make sure we have the required version of Pushover Notifications core plugin
		$plugin_folder = get_plugins( '/pushover-notifications' );
		$plugin_file = 'pushover-notifications.php';
		$core_version = $plugin_folder[$plugin_file]['Version'];
		$requires = '1.7.3.1';

		if ( version_compare( $core_version, $requires ) >= 0 ) {
			return true;
		}

		return false;
	}

	private function check_jetpack_active() {
		return is_plugin_active( 'jetpack/jetpack.php' );
	}

	private function check_stats_active() {
		$active_modules = Jetpack::get_active_modules();

		return in_array('stats', $active_modules);
	}


	public function load_text_domain() {
		load_plugin_textdomain( CKPN_JP_TEXT_DOMAIN, false, '/pushover-notifications-jp-ext/languages/' );
	}

	/*
	 * determine_cron_schedule
	 *
	 * Is used to figure out if our cron is already determined and then adds the hook for sending the daily stats
	 */
	public function determine_cron_schedule() {
		if ( !wp_next_scheduled( 'ckpn_jps_daily_stats' ) ) {
			$next_run = strtotime( '23:00' ) + ( -( get_option('gmt_offset') * 60 * 60 ) ); // Calc for the WP timezone

			if ( (int)date_i18n( 'G' ) >= 23 )
				$next_run = strtotime( 'next day 23:00' ) + ( -( get_option('gmt_offset') * 60 * 60 ) ); // Calc for the WP timezone;

			wp_schedule_event( $next_run, 'daily', 'ckpn_jps_daily_stats' );
		}
		add_action( 'ckpn_jps_daily_stats', array( $this, 'execute_daily_stats' ) );
	}

	public function execute_daily_stats() {
		$days_offset = ( get_option('gmt_offset') < 0 ) ? 2 : 1;
			
		$daily_page_views = stats_get_csv( 'postviews', 'days='.$days_offset.'&limit=0' ); // Jetpack Stats has a 5 minute cache on the keys

		$total_views = 0;
		foreach ( $daily_page_views as $page ) {
			$total_views += $page['views'];
		}

		$top_page_title = $daily_page_views[0]['post_title'];
		$top_page_views = $daily_page_views[0]['views'];

		$title = sprintf( __( '%s: Daily Stats', CKPN_JP_TEXT_DOMAIN ), get_bloginfo( 'name' ) );
		$message  = __( 'Top Page:', CKPN_JP_TEXT_DOMAIN );
		$message .= "\n" . $top_page_title . ' - ' . $top_page_views . __( ' views', CKPN_JP_TEXT_DOMAIN );
		$message .= "\n" . sprintf( __( 'Total Views: %d', CKPN_JP_TEXT_DOMAIN ), $total_views );

		$args = array( 'title' => $title, 'message' => $message );
		$this->send_notification( $args );
	}

	/*
	 * send_notification
	 *
	 * Instantiates the Core Plugin and sends the title and message to the core for sending push notifications
	 *
	 */
	private function send_notification( $args ) {
		$ckpn_core = CKPushoverNotifications::getInstance();
		$ckpn_core->ckpn_send_notification( $args );
	}


	/************************************************
	 * Nags about core being missing or out of date *
	 ************************************************/

	/*
	 * core_out_of_date_nag
	 *
	 * Warns a user if the core Pushover Notifications plugin is not at the minimum required version
	 *
	 *
	 * @return void
	 */
	function core_out_of_date_nag() {
		printf( '<div class="error"> <p> %s </p> </div>', esc_html__( 'Your Pushover Notifications core plugin is missing or out of date. Please update to at least version 1.7.3.1 in order to use Pushover Notifiations for Jetpack Stats.', CKPN_JP_TEXT_DOMAIN ) );
	}

	function jetpack_not_active() {
		printf( '<div class="error"> <p> %s </p> </div>', esc_html__( 'To use Pushover Notifications for Jetpack, you must activate the Jetpack Plugin.', CKPN_JP_TEXT_DOMAIN ) );
	}

	function stats_not_active() {
		printf( '<div class="error"> <p> %s </p> </div>', esc_html__( 'To receive Jetpack Stats via Pushover, you must enable the Jetpack Stats Module.', CKPN_JP_TEXT_DOMAIN ) );
	}
}

$ckpn_jp_loaded = CKPushoverNotificationsJP::getInstance();